"""Very simple kerberized socket interface.

Only stream sockets are supported by this interface.  There are no extra
methods that provide access to some of the facilities available on the
authentication context.  There is one new method, .kconnect(), which must
be used instead of the usual .connect().
"""
__version__ = '$Revision: 1.6 $'
#  $Source: /projects/cvsroot/goodies/krb5module/ksocket.py,v $

import krb5
import socket


class KSocketError(Exception):
    def __init__(self, message):
        self.message = message
        Exception.__init__(self, message)


class SocketProxy:
    """Implement a caching proxy pattern for Kerberized sockets."""

    _sock = None

    def __getattr__(self, name):
        if self._sock is not None:
            v = getattr(self._sock, name)
            self.__dict__[name] = v
            return v
        raise AttributeError, name

    def __del__(self):
        # the close() method must be able to handle multiple calls:
        self.close()


class KStreamSocket(SocketProxy):
    _ac = None
    _bound = 0

    def __init__(self, local_princ=None):
        if local_princ is None:
            local_princ = krb5.get_login_principal()
        elif not isinstance(local_princ, krb5.PrincipalType):
            raise TypeError, \
                  "optional parameter must be a Kerberos principal if given"
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._local_princ = local_princ

    def accept(self):
        sock, addr = self._sock.accept()
        ac = krb5.recv_auth(sock, self._local_princ)
        return KConnectionSocket(sock, ac), addr

    def bind(self, (host, port)):
        # this ensures we'll get the right IP address from getsockname()
        host = host or socket.gethostname()
        self._sock.bind((host, port))
        self._bound = 1

    def kconnect(self, sprinc, port, options=0):
        """Connect to a server.

        sprinc -- server principal
        port -- port number
        """
        if not isinstance(sprinc, krb5.PrincipalType):
            raise TypeError, "first parameter must be a service principal"
        if not (sprinc.host and sprinc.service):
            raise ValueError, "first parameter must be a service principal"
        if not self._bound:
            self.bind(('', 0))
        address = sprinc.host, port
        self._sock.connect(address)
        self._ac = krb5.send_auth(self._sock, self._local_princ, sprinc,
                                  options)

    def connect(self, address):
        raise KSocketError("cannot use connect() with kerberized sockets;"
                           " use kconnect() instead")

    def close(self):
        if self._sock:
            self._sock.close()
            self._sock = 0

    def get_remote_principal(self):
        if not self._ac:
            raise KSocketError("connection not established")
        return self._ac.remote


class KConnectionSocket(KStreamSocket):

    def __init__(self, sock, ac):
        self._sock = sock
        self._ac = ac
