#!/usr/bin/env python
#
# A Python version of the TCP sample simple client from the krb5 distribution
#
# A sample Kerberos server, which reads an AP_REQ from a TCP socket,
# decodes it, and writes back the results (in ASCII) to the client.
#
# Written by Roger E. Masse <rmasse@cnri.reston.va.us>

import os
import sys
import getopt
import socket
import krb5
import string

SIMPLE_SERVICE = "sample"

def usage(name):
    m = "usage:\t%s [-p port] [-s service] -h host\n" \
        % os.path.basename(name)
    sys.stderr.write(m)

def main():
    port = 0
    host = None
    reverse = 0
    service = SIMPLE_SERVICE
    try:
        opts, args = getopt.getopt(sys.argv[1:], "p:s:h:r")
    except getopt.error:
        usage()
        sys.exit(2)
    for opt, val in opts:
        if opt == "-p":
            port = string.atoi(val)
        elif opt == "-s":
            service = val
        elif opt == '-h':
            host = val
        elif opt == '-r':
            reverse = 1

    if not host:
        usage(sys.argv[0])
        sys.exit(2)

    # If user specified a port, try and connect using that port
    # otherwise look up the service and connect to its port
    if not port:
        port = socket.getservbyname(service, 'tcp')

    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.connect(host, port)
    print 'connected socket'

    cp = krb5.get_login_principal()
    print 'got login principal'

    sp = krb5.get_svc_principal(service)
    print 'got service principal'

    ac = krb5.send_auth(sock, cp, sp)
    print 'got authentication context'

    if reverse:
        ac.block_write("hello world")
        print 'wrote buf'

        ac.block_write_safe("hello safe world")
        print 'wrote safe buf'

        ac.block_write_priv("hello priv world")
        print 'wrote priv buf'
    else:
        while 1:
            try:
                buf = ac.block_read()
            except socket.error:
                break
            print 'block read: ', buf


if __name__ == '__main__':
    main()
