#!/usr/bin/env python
#
# A Python version of the TCP sample simple server from the krb5 distribution
#
# A sample Kerberos server, which reads an AP_REQ from a TCP socket,
# decodes it, and writes back the results (in ASCII) to the client.
#
# Written by Roger E. Masse <rmasse@cnri.reston.va.us>

import os
import sys
import getopt
import socket
import krb5
import string

SIMPLE_SERVICE = "sample"

def usage(name):
    m = "usage:\t%s [-p port] [-s service] [-r]\n" \
        % os.path.basename(name)
    sys.stderr.write(m)
    
def main():
    port = 5000
    host = ''   # local host
    reverse = 0
    service = SIMPLE_SERVICE
    try:
        opts, args = getopt.getopt(sys.argv[1:], "p:s:r")
    except getopt.error:
        usage()
        sys.exit(2)
    for opt, val in opts:
        if opt == "-p":
            port = string.atoi(val)
        elif opt == "-s":
            service = val
        elif opt == '-r':
            reverse = 1

    print 'port: ', port
    # If user specified a port, then listen on that port; otherwise,
    # assume we've been started out of inetd. 
    if port:
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        # Let the socket be reused right away
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.bind(host, port)
        sock.listen(1)
        print 'about to accept'
        sock, addr = sock.accept()
        print 'accepted'
    else:
        # Moving the TCPAuthenticationContext.set_address() and .set_rcache()
        # to happen automatically in C should make this problem go away.  Not
        # sure that much can be done for the UDP version yet, but perhaps.
        raise RuntimeError, "this doesn't work from inet.d for Python, yet"

    # Ready to use Kerberos get the principal for the service
    sp = krb5.get_svc_principal(service)
    print 'got service principal'

    # Get a krb5 TCP authentication context
    ac = krb5.recv_auth(sock, sp)
    print 'got authentication context'

    if reverse:
        while 1:
            try:
                buf = ac.block_read()
            except socket.error:
                break
            print 'block read: ', buf
    else:
        ac.block_write("hello world")
        print 'wrote buf'

        ac.block_write_safe("hello safe world")
        print 'wrote safe buf'

        ac.block_write_priv("hello priv world")
        print 'wrote priv buf'


if __name__ == '__main__':
    main()
