#! /usr/bin/env python

"""Python implementation of the sim_client program from Kerberos 5."""

__version__ = '$Revision: 1.3 $'
#  $Source: /projects/cvsroot/goodies/krb5module/samples/udpclient.py,v $

import getopt
import krb5
import os
import socket
import string
import sys


BUFSIZ = 1024
SIMPLE_SERVICE = "sample"
SIMPLE_PORT = SIMPLE_SERVICE

MSG = "hi there!"


def usage(name):
    m = "usage:\t%s [-p port] [-h host] [-m message] [-s service]\n" \
        % os.path.basename(name)
    sys.stderr.write(m)


def main():
    hostname = None
    port = 0
    service = SIMPLE_SERVICE
    message = MSG
    try:
        opts, args = getopt.getopt(sys.argv[1:], "p:h:m:s:")
    except getopt.error:
        usage(sys.argv[0])
        sys.exit(2)
    for opt, val in opts:
        if opt == "-p":
            port = string.atoi(val)
        elif opt == "-h":
            hostname = val
        elif opt == "-m":
            message = val
        elif opt == "-s":
            service = val
    if hostname and args:
        usage(sys.argv[0])
        sys.exit(2)
    try:
        hostname = hostname or args[0]
    except IndexError:
        usage(sys.argv[0])
        sys.stderr.write("\nYou must specify a hostname to contact.\n\n")
        sys.exit(2)
    if not port:
        port = socket.getservbyname(SIMPLE_PORT, "udp")
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    # bind() is used because we're using connected datagram sockets. ???
    # This is also needed to support subsequent calls to .getsockname().
    localip = socket.gethostbyname(socket.gethostname())
    sock.bind((localip, 0))
    print sock.getsockname()
    sprinc = krb5.get_svc_principal(service, hostname)
    auth_context, packet = krb5.mk_req(
        sprinc, sock.getsockname(), (hostname, port))
    print "Got credentials for %s." % service

    i = sock.sendto(packet, (hostname, port))
    print "Sent authentication data:", i, "bytes"

    packet = auth_context.mk_safe(message)
    i = sock.sendto(packet, (hostname, port))
    print "Sent checksummed message:", i, "bytes"

    packet = auth_context.mk_priv(message)
    i = sock.sendto(packet, (hostname, port))
    print "Sent encrypted message:", i, "bytes"


if __name__ == "__main__":
    main()
